//
// Helper functions for TRT N-Tuple analysis framework - all those relating to hit-level objects
//

bool indet_isHT(TRT *Trk, unsigned int ihit) { // HLP
  unsigned int word = Trk->InDetTrackParticles_hit_bitPattern->at(ihit);
  return (word & 0x00020000u);
}

bool gsf_isHT(TRT *Trk, unsigned int ihit) { // HLP
  unsigned int word = Trk->GSFTrackParticles_hit_bitPattern->at(ihit);
  return (word & 0x00020000u);
}

bool is_any_ht_bit_set(unsigned int word) { // HLP
  return (word & 0x04020100u);
}

bool is_middle_ht_bit_set(unsigned int word) { // HLP
  return (word & 0x00020000u);
}

void set_ht_bit(unsigned int &word) { // HLP
  word = word | 0x00020000u;
}

void print_bit_pattern(unsigned int word) { // HLP
/*
  Print out the bit pattern.
  In (recent) MC the following bits are zeroed:
  msb, and first 4 unused bits, first and third HT bits, last 4 LT bits
  The msb is set to one if it is MC overlayed.

   bit31                          bit0
   |    leading          trailing |
   #----HLLLLLLLLHLLLLLLLLHLLLLLLLL
   00000011111111111111111011110000
*/
  cout << "{";
  for (unsigned int k=0; k<32; k++) {
    unsigned int i = word & 0x80000000u;
    bool b = (i==0x80000000u);
    if (!((k-5)%9)) std::cout << " ";
    cout << b;
    if (!((k-5)%9)) std::cout << " ";
    word=word<<1;
  }
  cout << "}"<< endl;
}

unsigned int get_bit_pattern_bit(unsigned int word, unsigned int k) { // HLP
  word=word<<k;
  unsigned int i = word & 0x80000000u;
  bool b = (i==0x80000000u);
  return b;
}

/*
  A collection of helpers related to hit types and hit counts on a track.
  Boolean definitions of hit types (precision, tube, outlier) are such that on
  a track #isPrecise + #isTubehit + #isOutlier is equal to the hit vector size.
  This requires isPrecise to exclude precision hits that are also outliers.

  Notes:

    * Trk->InDetTrackParticles_nTRTHits is the number of non-outlier hits.
    * about 1% of precision hits are also outliers; so only use
      Trk->InDetTrackParticles_nTRTHitsPrecision if you intend to include precise outliers
    * The sum of isOutlier hits agrees with Trk->InDetTrackParticles_nTRTOutliers;

*/
bool indet_isOutlier(TRT *Trk, unsigned int ihit) { // HLP
  return Trk->InDetTrackParticles_hit_type->at(ihit)==5; // 0="precision or tube"
}
bool indet_isTubehit(TRT *Trk, unsigned int ihit) { // HLP
  return Trk->InDetTrackParticles_hit_type->at(ihit)==0 && !Trk->InDetTrackParticles_hit_isPrecision->at(ihit);
}
bool indet_isPrecise(TRT *Trk, unsigned int ihit) { // HLP
  // not including precision hits that are also outliers
  return Trk->InDetTrackParticles_hit_isPrecision->at(ihit) && Trk->InDetTrackParticles_hit_type->at(ihit)==0;
}
bool gsf_isOutlier(TRT *Trk, unsigned int ihit) { // HLP
  return Trk->GSFTrackParticles_hit_type->at(ihit)==5; // 0="precision or tube"
}
bool gsf_isTubehit(TRT *Trk, unsigned int ihit) { // HLP
  return Trk->GSFTrackParticles_hit_type->at(ihit)==0 && !Trk->GSFTrackParticles_hit_isPrecision->at(ihit);
}
bool gsf_isPrecise(TRT *Trk, unsigned int ihit) { // HLP
  // not including precision hits that are also outliers
  return Trk->GSFTrackParticles_hit_isPrecision->at(ihit) && Trk->GSFTrackParticles_hit_type->at(ihit)==0;
}
unsigned int indet_nOutlier(TRT *Trk) { // HLP
  unsigned int n=0;
  for (unsigned int ihit=0; ihit<Trk->InDetTrackParticles_hit_type->size(); ihit++) {
    if ( indet_isOutlier(Trk, ihit) ) n++;
  }
  if (n!=Trk->InDetTrackParticles_nTRTOutliers) WARN("nOutlier() count does not agree with _nTRTOutliers!");
  return n;
}
unsigned int indet_nTubehit(TRT *Trk) { // HLP
  unsigned int n=0;
  for (unsigned int ihit=0; ihit<Trk->InDetTrackParticles_hit_type->size(); ihit++) {
    if ( indet_isTubehit(Trk, ihit) ) n++;
  }
  return n;
}
unsigned int indet_nPrecise(TRT *Trk) { // HLP
  unsigned int n=0;
  for (unsigned int ihit=0; ihit<Trk->InDetTrackParticles_hit_type->size(); ihit++) {
    if ( indet_isPrecise(Trk, ihit) ) n++;
  }
  return n;
}
unsigned int gsf_nOutlier(TRT *Trk) { // HLP
  unsigned int n=0;
  for (unsigned int ihit=0; ihit<Trk->GSFTrackParticles_hit_type->size(); ihit++) {
    if ( gsf_isOutlier(Trk, ihit) ) n++;
  }
  if (n!=Trk->GSFTrackParticles_nTRTOutliers) WARN("nOutlier() count does not agree with _nTRTOutliers!");
  return n;
}
unsigned int gsf_nTubehit(TRT *Trk) { // HLP
  unsigned int n=0;
  for (unsigned int ihit=0; ihit<Trk->GSFTrackParticles_hit_type->size(); ihit++) {
    if ( gsf_isTubehit(Trk, ihit) ) n++;
  }
  return n;
}
unsigned int gsf_nPrecise(TRT *Trk) { // HLP
  unsigned int n=0;
  for (unsigned int ihit=0; ihit<Trk->GSFTrackParticles_hit_type->size(); ihit++) {
    if ( gsf_isPrecise(Trk, ihit) ) n++;
  }
  return n;
}

bool indet_isXenon(TRT *Trk, unsigned int ihit) { // HLP
  int gas = Trk->InDetTrackParticles_hit_gasType->at(ihit);
  if ( gas!=0 && gas!=1 ) ERROR("Unknown gas in indet_is_xenon().", gas);
  if ( gas==0 ) return true;
  return false;
}
bool indet_isArgon(TRT *Trk, unsigned int ihit) { // HLP
  int gas = Trk->InDetTrackParticles_hit_gasType->at(ihit);
  if ( gas!=0 && gas!=1 ) ERROR("Unknown gas in indet_is_argon().", gas);
  if ( gas==1 ) return true;
  return false;
}
bool gsf_isXenon(TRT *Trk, unsigned int ihit) { // HLP
  int gas = Trk->GSFTrackParticles_hit_gasType->at(ihit);
  if ( gas!=0 && gas!=1 ) ERROR("Unknown gas in gsf_is_xenon().", gas);
  if ( gas==0 ) return true;
  return false;
}
bool gsf_isArgon(TRT *Trk, unsigned int ihit) { // HLP
  int gas = Trk->GSFTrackParticles_hit_gasType->at(ihit);
  if ( gas!=0 && gas!=1 ) ERROR("Unknown gas in gsf_is_argon().", gas);
  if ( gas==1 ) return true;
  return false;
}

unsigned int dumpgas[2][34]={{0}}; // HLP
/* Gas geometry
   Insert  dump_gas_add(Trk);  inside your track loop
   Insert  dump_gas_show();    after your track loop
*/

void dump_gas_increment(TRT *Trk) { // HLP
  /*
    Loop over the straws of this InDet track and increment
    the gas count for the (transformed) layer
  */
  for ( unsigned int ihit=0; ihit<Trk->InDetTrackParticles_hit_gasType->size(); ihit++ ) {

    int gas = Trk->InDetTrackParticles_hit_gasType->at(ihit);
    if ( gas!=0 && gas!=1 ) {
      WARN("Unknown gas in dump_gas_add().");
      continue;
    }

    int bec   = Trk->InDetTrackParticles_hit_bec->at(ihit);
    int layer = Trk->InDetTrackParticles_hit_layer->at(ihit);

    int k=-1;
    if      ( bec==-2 ) k=13-layer;
    else if ( bec==-1 ) k=16-layer;
    else if ( bec==+1 ) k=17+layer;
    else if ( bec==+2 ) k=20+layer;
    else WARN("Bad bec value in dump_gas_add().");

    if ( k<0 || k>33 ) {
      WARN("Bad dumpgas index in dump_gas_add().");
      continue;
    }

    dumpgas[gas][k]++;

  } // end of straw loop
}

void dump_gas_show() { // HLP

  string color="";
  string element="";
  cout << endl << CYellow << " Gas Geometry" << CNone << endl;
  //cout << " 3 2 1 0 9 8 7 6  5 4 3 2 1 0                     0 1 2 3 4 5  6 7 8 9 0 1 2 3" << endl;
  for ( unsigned int j=0; j<3; j++ ) {
    for ( unsigned int k=0; k<34; k++ ) {
      if  (dumpgas[0][k]>0 && dumpgas[1][k]>0)
        WARN("Array dumpgas has both gas types in the same module.");
      else if ( dumpgas[0][k]>0 ) color = CGreen;
      else if ( dumpgas[1][k]>0 ) color = CBlue;
      else                        color = CNone;
      if ( k<14 || k>19 ) {
        element="I";
        cout << " " << color << element;
      } else {
        element="========";
        if ( k==14 && j==0 ) cout << " " << color << element << CNone << CGrey << 2 << CNone;
        if ( k==15 && j==1 ) cout << " " << color << element << CNone << CGrey << 1 << CNone;
        if ( k==16 && j==2 ) cout << " " << color << element << CNone << CGrey << 0 << CNone;
        if ( k==17 && j==2 ) cout << color << element;
        if ( k==18 && j==1 ) cout << color << element;
        if ( k==19 && j==0 ) cout << color << element;
      }
      if ( k==7 || k==13 || k==19 || k==25 ) cout << " ";
    }
    cout << CNone << endl;
  }
  cout << CGrey << " 3 2 1 0 9 8 7 6  5 4 3 2 1 0                     0 1 2 3 4 5  6 7 8 9 0 1 2 3" << CNone <<endl;

  /* Run2 scenario 5
    B0,B1
    EA3,EA5
    EC3,EC5,EC8
  */
  string s_run2=" TrtStrawStatusHT-RUN2-scenario5-00-00; Argon: B0,B1, EA3,EA5, EC3,EC5,EC8; Used during 2016 data taking.";
  bool b_run2=true;
  if ( !dumpgas[0][0]  ||  dumpgas[1][0] )  b_run2=false;
  if ( !dumpgas[0][1]  ||  dumpgas[1][1] )  b_run2=false;
  if ( !dumpgas[0][2]  ||  dumpgas[1][2] )  b_run2=false;
  if ( !dumpgas[0][3]  ||  dumpgas[1][3] )  b_run2=false;
  if ( !dumpgas[0][4]  ||  dumpgas[1][4] )  b_run2=false;
  if (  dumpgas[0][5]  || !dumpgas[1][5] )  b_run2=false;
  if ( !dumpgas[0][6]  ||  dumpgas[1][6] )  b_run2=false;
  if ( !dumpgas[0][7]  ||  dumpgas[1][7] )  b_run2=false;
  if (  dumpgas[0][8]  || !dumpgas[1][8] )  b_run2=false;
  if ( !dumpgas[0][9]  ||  dumpgas[1][9] )  b_run2=false;
  if (  dumpgas[0][10] || !dumpgas[1][10] ) b_run2=false;
  if ( !dumpgas[0][11] ||  dumpgas[1][11] ) b_run2=false;
  if ( !dumpgas[0][12] ||  dumpgas[1][12] ) b_run2=false;
  if ( !dumpgas[0][13] ||  dumpgas[1][13] ) b_run2=false;
  if ( !dumpgas[0][14] ||  dumpgas[1][14] ) b_run2=false;
  if (  dumpgas[0][15] || !dumpgas[1][15] ) b_run2=false;
  if (  dumpgas[0][16] || !dumpgas[1][16] ) b_run2=false;
  if (  dumpgas[0][17] || !dumpgas[1][17] ) b_run2=false;
  if (  dumpgas[0][18] || !dumpgas[1][18] ) b_run2=false;
  if ( !dumpgas[0][19] ||  dumpgas[1][19] ) b_run2=false;
  if ( !dumpgas[0][20] ||  dumpgas[1][20] ) b_run2=false;
  if ( !dumpgas[0][21] ||  dumpgas[1][21] ) b_run2=false;
  if ( !dumpgas[0][22] ||  dumpgas[1][22] ) b_run2=false;
  if (  dumpgas[0][23] || !dumpgas[1][23] ) b_run2=false;
  if ( !dumpgas[0][24] ||  dumpgas[1][24] ) b_run2=false;
  if (  dumpgas[0][25] || !dumpgas[1][25] ) b_run2=false;
  if ( !dumpgas[0][26] ||  dumpgas[1][26] ) b_run2=false;
  if ( !dumpgas[0][27] ||  dumpgas[1][27] ) b_run2=false;
  if ( !dumpgas[0][28] ||  dumpgas[1][28] ) b_run2=false;
  if ( !dumpgas[0][29] ||  dumpgas[1][29] ) b_run2=false;
  if ( !dumpgas[0][30] ||  dumpgas[1][30] ) b_run2=false;
  if ( !dumpgas[0][31] ||  dumpgas[1][31] ) b_run2=false;
  if ( !dumpgas[0][32] ||  dumpgas[1][32] ) b_run2=false;
  if ( !dumpgas[0][33] ||  dumpgas[1][33] ) b_run2=false;
  if ( b_run2 ) cout << CGrey << s_run2 << CNone << endl;

  /* Run3 run numbers > 330000
    Full Barrel (B0,B1,B2)
    EA3,EA5,
    EC0,EC3,EC5,EC8,EC10
  */
  string s_run2run3=" TrtStrawStatusHT-MC-run2-run3-01; Run3 scenario run numbers 330000-inf: Full Barrel, EA3,EA5, EC0,EC3,EC5,EC8,EC10";
  bool b_run2run3=true;
  if ( !dumpgas[0][0]  ||  dumpgas[1][0] )  b_run2run3=false;
  if ( !dumpgas[0][1]  ||  dumpgas[1][1] )  b_run2run3=false;
  if ( !dumpgas[0][2]  ||  dumpgas[1][2] )  b_run2run3=false;
  if (  dumpgas[0][3]  || !dumpgas[1][3] )  b_run2run3=false;
  if ( !dumpgas[0][4]  ||  dumpgas[1][4] )  b_run2run3=false;
  if (  dumpgas[0][5]  || !dumpgas[1][5] )  b_run2run3=false;
  if ( !dumpgas[0][6]  ||  dumpgas[1][6] )  b_run2run3=false;
  if ( !dumpgas[0][7]  ||  dumpgas[1][7] )  b_run2run3=false;
  if (  dumpgas[0][8]  || !dumpgas[1][8] )  b_run2run3=false;
  if ( !dumpgas[0][9]  ||  dumpgas[1][9] )  b_run2run3=false;
  if (  dumpgas[0][10] || !dumpgas[1][10] ) b_run2run3=false;
  if ( !dumpgas[0][11] ||  dumpgas[1][11] ) b_run2run3=false;
  if ( !dumpgas[0][12] ||  dumpgas[1][12] ) b_run2run3=false;
  if (  dumpgas[0][13] || !dumpgas[1][13] ) b_run2run3=false;
  if (  dumpgas[0][14] || !dumpgas[1][14] ) b_run2run3=false;
  if (  dumpgas[0][15] || !dumpgas[1][15] ) b_run2run3=false;
  if (  dumpgas[0][16] || !dumpgas[1][16] ) b_run2run3=false;
  if (  dumpgas[0][17] || !dumpgas[1][17] ) b_run2run3=false;
  if (  dumpgas[0][18] || !dumpgas[1][18] ) b_run2run3=false;
  if (  dumpgas[0][19] || !dumpgas[1][19] ) b_run2run3=false;
  if ( !dumpgas[0][20] ||  dumpgas[1][20] ) b_run2run3=false;
  if ( !dumpgas[0][21] ||  dumpgas[1][21] ) b_run2run3=false;
  if ( !dumpgas[0][22] ||  dumpgas[1][22] ) b_run2run3=false;
  if (  dumpgas[0][23] || !dumpgas[1][23] ) b_run2run3=false;
  if ( !dumpgas[0][24] ||  dumpgas[1][24] ) b_run2run3=false;
  if (  dumpgas[0][25] || !dumpgas[1][25] ) b_run2run3=false;
  if ( !dumpgas[0][26] ||  dumpgas[1][26] ) b_run2run3=false;
  if ( !dumpgas[0][27] ||  dumpgas[1][27] ) b_run2run3=false;
  if ( !dumpgas[0][28] ||  dumpgas[1][28] ) b_run2run3=false;
  if ( !dumpgas[0][29] ||  dumpgas[1][29] ) b_run2run3=false;
  if ( !dumpgas[0][30] ||  dumpgas[1][30] ) b_run2run3=false;
  if ( !dumpgas[0][31] ||  dumpgas[1][31] ) b_run2run3=false;
  if ( !dumpgas[0][32] ||  dumpgas[1][32] ) b_run2run3=false;
  if ( !dumpgas[0][33] ||  dumpgas[1][33] ) b_run2run3=false;
  if ( b_run2run3 ) cout << CGrey << s_run2run3 << CNone << endl;

  cout << endl;

}

unsigned int indet_hit_vector_size(TRT *Trk) { // HLP
  /*
    Return the size of the InDetTrackParticles_hit_* vectors
    This also issues an warning if all vectors are not the same size.
    Add/remove variables as needed when the ntuple content changes.
  */
  bool err=false;
  unsigned int n = Trk->InDetTrackParticles_hit_type->size();
  if ( Trk->InDetTrackParticles_hit_type->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_r->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_z->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_localTheta->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_localPhi->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_rTrackToWire->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_unbiasedResidual->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_biasedResidual->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_unbiasedPull->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_biasedPull->size()!=n ) err=true;
  //if ( Trk->InDetTrackParticles_hit_isHighThreshold->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_gasType->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_bec->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_layer->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_strawLayer->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_strawNumber->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_board->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_chip->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_modulePhi->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_bitPattern->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_strawPhi->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_t0->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_leadingEdge->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_driftTime->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_driftRadius->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_tot->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_isPrecision->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_strawPosition->size()!=n ) err=true;
  if ( Trk->InDetTrackParticles_hit_l->size()!=n ) err=true;
  if ( err ) WARN("InDetTrackParticles_hit_* not all the same for this track in indet_hit_vector_size().");
  return n;
}

unsigned int gsf_hit_vector_size(TRT *Trk) { // HLP
  /*
    Return the size of the GSFTrackParticles_hit_* vectors
    This also issues an warning if all vectors are not the same size.
    Add/remove variables as needed when the ntuple content changes.
  */
  bool err=false;
  unsigned int n = Trk->GSFTrackParticles_hit_type->size();
  if ( Trk->GSFTrackParticles_hit_type->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_r->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_z->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_localTheta->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_localPhi->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_rTrackToWire->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_unbiasedResidual->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_biasedResidual->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_unbiasedPull->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_biasedPull->size()!=n ) err=true;
  //if ( Trk->GSFTrackParticles_hit_isHighThreshold->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_gasType->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_bec->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_layer->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_strawLayer->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_strawNumber->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_board->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_chip->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_modulePhi->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_bitPattern->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_strawPhi->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_t0->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_leadingEdge->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_driftTime->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_driftRadius->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_tot->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_isPrecision->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_strawPosition->size()!=n ) err=true;
  if ( Trk->GSFTrackParticles_hit_l->size()!=n ) err=true;
  if ( err ) WARN("GSFTrackParticles_hit_* not all the same for this track in gsf_hit_vector_size().");
  return n;
}

/////////////////////////////////////////////
// Suite of "straw layer number" functions //
/////////////////////////////////////////////
/*
    _hit_bec        - Barrel-Endcap {-2,-1,1,2}
    _hit_layer      - module layer in bec: Barrel [3], Endcap [14]
    _hit_strawLayer - Straw layer in module layer: Barrel [19, 24, 30], Endcap [6x16, 8x8]
*/

unsigned int straw_layer_number(int bec, int layer, int straw) { // HLP
  /*
    Return the proper straw layer number of TRT *Trk using _hit_bec, _hit_later, _hit_strawLayer.
    That is, 0-72 for the barrel, and 0-159 for the endcap.
    This assumes you are paying attention to which _hit_bec region of the detector you are referring to.
  */
  unsigned int sln=straw;
  if ( abs(bec)==1 ) {
    if ( layer>0 ) sln += 19;
    if ( layer>1 ) sln += 24;
  } else if ( abs(bec)==2 ) {
    if ( layer<6 )  sln += layer*16;
    if ( layer>=6 ) sln += 6*16 + (layer-6)*8;
  }
  else WARN("Bad bec value in straw_layer_number().");
  if ( abs(bec)==1 && sln>72 )  WARN("Out of range straw_layer_number for the barrel in straw_layer_number().");
  if ( abs(bec)==2 && sln>159 ) WARN("Out of range straw_layer_number for the end-cap in straw_layer_number().");
  return sln;
}

unsigned int indet_straw_layer_number(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  int layer = Trk->InDetTrackParticles_hit_layer->at(ihit);
  int straw = Trk->InDetTrackParticles_hit_strawLayer->at(ihit);
  return straw_layer_number(bec, layer, straw);
}

unsigned int gsf_straw_layer_number(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->GSFTrackParticles_hit_bec->at(ihit);
  int layer = Trk->GSFTrackParticles_hit_layer->at(ihit);
  int straw = Trk->GSFTrackParticles_hit_strawLayer->at(ihit);
  return straw_layer_number(bec, layer, straw);
}

//////////////////////////////////////////////////////
// End of - Suite of "straw layer number" functions //
//////////////////////////////////////////////////////

/////////////////////////////////////////
// Suite of "region boolean" functions //
/////////////////////////////////////////

bool isSideA(int bec) { // HLP
  if (bec>0) return true;
  if (bec<0) return false;
  WARN("Bad bec value in isSideA()", bec);
  return false;
}
bool isSideC(int bec) { // HLP
  if (bec<0) return true;
  if (bec>0) return false;
  WARN("Bad bec value in isSideC()", bec);
  return false;
}

bool isBarrel(int bec) { // HLP
  if (abs(bec)==1) return true;
  if (abs(bec)==2) return false;
  WARN("Bad bec value in isBarrel()", bec);
  return false;
}
bool isEndCap(int bec) { // HLP
  if (abs(bec)==2) return true;
  if (abs(bec)==1) return false;
  WARN("Bad bec value in isEndcap()", bec);
  return false;
}

bool isBarrelLayer0(int bec, int layer) { // HLP
  if (isEndCap(bec)) return false;
  if (layer<0 || layer>2) WARN("Bad bec value in isBarrelLayer0()", layer);
  if (layer==0) return true;
  return false;
}
bool isBarrelLayer1(int bec, int layer) { // HLP
  if (isEndCap(bec)) return false;
  if (layer<0 || layer>2) WARN("Bad bec value in isBarrelLayer1()", layer);
  if (layer==1) return true;
  return false;
}
bool isBarrelLayer2(int bec, int layer) { // HLP
  if (isEndCap(bec)) return false;
  if (layer<0 || layer>2) WARN("Bad bec value in isBarrelLayer2()", layer);
  if (layer==2) return true;
  return false;
}

bool isEndCapWheelA(int bec, int layer) { // HLP
  if (isBarrel(bec)) return false;
  if (layer<0 || layer>13) WARN("Bad bec value in isEndcapWheelA()", layer);
  if (layer<6) return true;
  return false;
}
bool isEndCapWheelB(int bec, int layer) { // HLP
  if (isBarrel(bec)) return false;
  if (layer<0 || layer>13) WARN("Bad bec value in isEndcapWheelA()", layer);
  if (layer>5) return true;
  return false;
}

// InDetTracks ...

// Sides
bool indet_isSideA(TRT *Trk, unsigned int ihit) { // HLP
  int bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  return isSideA(bec);
}
bool indet_isSideC(TRT *Trk, unsigned int ihit) { // HLP
  int bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  return isSideC(bec);
}

// Barrel
bool indet_isBarrel(TRT *Trk, unsigned int ihit) { // HLP
  int bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  return isBarrel(bec);
}
bool indet_isBarrelLayer0(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  int layer = Trk->InDetTrackParticles_hit_layer->at(ihit);
  return isBarrelLayer0(bec, layer);
}
bool indet_isBarrelLayer1(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  int layer = Trk->InDetTrackParticles_hit_layer->at(ihit);
  return isBarrelLayer1(bec, layer);
}
bool indet_isBarrelLayer2(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  int layer = Trk->InDetTrackParticles_hit_layer->at(ihit);
  return isBarrelLayer2(bec, layer);
}

// End-cap
bool indet_isEndCap(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  return isEndCap(bec);
}
bool indet_isEndCapWheelA(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  int layer = Trk->InDetTrackParticles_hit_layer->at(ihit);
  return isEndCapWheelA(bec, layer);
}
bool indet_isEndCapWheelB(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  int layer = Trk->InDetTrackParticles_hit_layer->at(ihit);
  return isEndCapWheelB(bec, layer);
}

// GSFTracks ...

// Sides
bool gsf_isSideA(TRT *Trk, unsigned int ihit) { // HLP
  int bec = Trk->GSFTrackParticles_hit_bec->at(ihit);
  return isSideA(bec);
}
bool gsf_isSideC(TRT *Trk, unsigned int ihit) { // HLP
  int bec = Trk->GSFTrackParticles_hit_bec->at(ihit);
  return isSideC(bec);
}

// Barrel
bool gsf_isBarrel(TRT *Trk, unsigned int ihit) { // HLP
  int bec = Trk->GSFTrackParticles_hit_bec->at(ihit);
  return isBarrel(bec);
}
bool gsf_isBarrelLayer0(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->GSFTrackParticles_hit_bec->at(ihit);
  int layer = Trk->GSFTrackParticles_hit_layer->at(ihit);
  return isBarrelLayer0(bec, layer);
}
bool gsf_isBarrelLayer1(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->GSFTrackParticles_hit_bec->at(ihit);
  int layer = Trk->GSFTrackParticles_hit_layer->at(ihit);
  return isBarrelLayer1(bec, layer);
}
bool gsf_isBarrelLayer2(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->GSFTrackParticles_hit_bec->at(ihit);
  int layer = Trk->GSFTrackParticles_hit_layer->at(ihit);
  return isBarrelLayer2(bec, layer);
}

// End-cap
bool gsf_isEndCap(TRT *Trk, unsigned int ihit) { // HLP
  int bec = Trk->GSFTrackParticles_hit_bec->at(ihit);
  return isEndCap(bec);
}
bool gsf_isEndCapWheelA(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->GSFTrackParticles_hit_bec->at(ihit);
  int layer = Trk->GSFTrackParticles_hit_layer->at(ihit);
  return isEndCapWheelA(bec, layer);
}
bool gsf_isEndCapWheelB(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->GSFTrackParticles_hit_bec->at(ihit);
  int layer = Trk->GSFTrackParticles_hit_layer->at(ihit);
  return isEndCapWheelB(bec, layer);
}

///////////////////////////////////////////////
//End of Suite of "region boolean" functions //
///////////////////////////////////////////////

/*
  6 TRT partitions, for partition occupancy.
  Return values; positive for side A, negative for side C
  0:undetermined: 1:Barrel (3 layers): 2:Endcap wheel A (6 layers): 3:Endcap wheel B (8 layers)
*/

int indet_partition(TRT *Trk, unsigned int ihit) { // HLP
  int bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  if (abs(bec)==1) return bec; // Barrel A,C
  if (bec==0 || abs(bec)>2) {
    WARN("Bad bec value in indet_partition().");
    return 0;
  }
  int layer = Trk->InDetTrackParticles_hit_layer->at(ihit);
  if (layer<0) {
    WARN("Bad layer value in indet_partition().");
    return 0;
  }
  if (bec>0 && layer<6) return +2; // Endcap A Wheel A
  if (bec>0 && layer>5) return +3; // Endcap A Wheel B
  if (bec<0 && layer<6) return -2; // Endcap C Wheel A
  if (bec<0 && layer>5) return -3; // Endcap C wheel B
  WARN("Bad partition value in indet_partition().");
  return 0;
}

int gsf_partition(TRT *Trk, unsigned int ihit) { // HLP
  int bec = Trk->GSFTrackParticles_hit_bec->at(ihit);
  if (abs(bec)==1) return bec; // Barrel A,C
  if (bec==0 || abs(bec)>2) {
    WARN("Bad bec value in gsf_partition().");
    return 0;
  }
  int layer = Trk->GSFTrackParticles_hit_layer->at(ihit);
  if (layer<0) {
    WARN("Bad layer value in gsf_partition().");
    return 0;
  }
  if (bec>0 && layer<6) return +2; // Endcap A Wheel A
  if (bec>0 && layer>5) return +3; // Endcap A Wheel B
  if (bec<0 && layer<6) return -2; // Endcap C Wheel A
  if (bec<0 && layer>5) return -3; // Endcap C wheel B
  WARN("Bad partition value in gsf_partition().");
  return 0;
}
