//---------------------------------------------------------------------------------------------------
// PlotFits.C
//
// This root macro file gets residual histograms from "histograms.root" and fits them to
// gaussian functions to obtain mean and std of the distributions.
// The mean and std values are printed to a data file called "data.txt".
//
// Last update: Dec 2022
// Author     : Ahmet.Bingul@cern.ch
//---------------------------------------------------------------------------------------------------

#include "../../TrtBigPaper/TrtBigPaper.cpp"
#include "../includes/TRT.C"
#include "../includes/helpers.cpp"

void PlotFits() {
   // input root file
   TFile *rootfile = new TFile("../histograms.root");
   // the canvas
   TCanvas *c1 = new TCanvas("c1","c1",0,0,600,600);
   // output data file
   ofstream fout1("data1.txt");
   ofstream fout2("data2.txt");
   ofstream fout3("data3.txt");
   fout1 << rootfile->GetName() << endl;
   fout2 << rootfile->GetName() << endl;

   TNamed *NTupleFile1 = (TNamed*) rootfile->Get("NTupleFile1");
   TNamed *NTupleFile2 = (TNamed*) rootfile->Get("NTupleFile2");
   fout3 << NTupleFile1->GetTitle() << endl;
   fout3 << NTupleFile2->GetTitle() << endl;

   // get list of histograms in root file ------------
   TIter   keyList(rootfile->GetListOfKeys());
   TKey    *key;

   while ( (key = (TKey*)keyList()) )
   {
      TClass *cl = gROOT->GetClass(key->GetClassName());
      if (!cl->InheritsFrom("TH1")) continue;
      // get histograms
      TH1D *h = (TH1D*) key->ReadObj();

      // consider only histograms starting with hRes
      string name = h->GetName();
      string head = name.substr(0,4);
      char tail = name[name.length()-1];
      int  sample;
      if(tail=='1') sample = 1;
      if(tail=='2') sample = 2;
      if(head!="hRes") continue;

      // draw histogram
      c1->Clear();
      h->Draw();

      // get mean ad std of residuals via iterative gaussian function
      double mean=0, sigma=0, meanError=0, sigmaError=0;
      iterGaus(h, 1.5, 1.0e-4, 10, false, mean, meanError, sigma, sigmaError);
      // print results to data files
      if(sample==1) fout1 << h->GetName() << "\t" << mean  << " " << meanError << "     "<< sigma << " " << sigmaError << endl;
      if(sample==2) fout2 << h->GetName() << "\t" << mean  << " " << meanError << "     "<< sigma << " " << sigmaError << endl;
      // generate png files
      string foldername  = "PngFits/";
      string pngfile = foldername+h->GetName()+".png";
      c1->Print(pngfile.c_str());
   }
   // ------------------------------------------------
   fout1.close();
   fout2.close();

   cout << "ALL OK."<< endl;
}
