//---------------------------------------------------------------------------------------------------
// PlotResiduals.C
//
// This root macro file reads mean and std of residuals from data file generated by PlotFits.C.
// Plot residuals as function for Track Occupancy and detector, BA, BC, EAA, ... etc.
//
// Last update: Dec 2022
// Author     : Ahmet.Bingul@cern.ch
//---------------------------------------------------------------------------------------------------

#include "AtlasStyle.C"
#include "../../TrtBigPaper/TrtBigPaper.cpp"
#include "../includes/TRT.C"
#include "../includes/helpers.cpp"

// -----------------------------------------------------------------------------------
void PlotResiduals(){
  gROOT->Reset();
  SetAtlasStyle();
  // we have 6 detectors and 2 gas types (6*2=12) configurations
  const int m = 12;
  // we have 10 track occupancy intervals
  const int n = 10;

  //TCanvas      *c1 = new TCanvas("c1","c1",0,0,800,600);
  TGraphErrors *gr1, *gr2;
  TH1D         *hist1, *hist2;

  ifstream fin1("data1.txt");
  ifstream fin2("data2.txt");
  string histofile, key1, key2, Key1[m], Key2[m];
  double occ[n]    = {0}, occError[n]   = {0};
  double mean1[n]  = {0}, meanError1[n] = {0};
  double mean2[n]  = {0}, meanError2[n] = {0};
  double sigma1[n] = {0}, sigmaError1[n]= {0};
  double sigma2[n] = {0}, sigmaError2[n]= {0};

  // Andrew's TrtBigPaperOptions
  TrtBigPaperOptions *h_opt = new TrtBigPaperOptions();
  h_opt->status    = "Internal";
  h_opt->simu      = "Simulation";
  h_opt->data      = "Data";
  h_opt->x_title   = "TRT Track Occupancy";
  h_opt->y_title   = "Position Accuracy [mm]";
  h_opt->entriesbw = false;
  h_opt->statbox   = false;
  h_opt->ratio     = true;
  h_opt->showinfo  = false;
  h_opt->min       = 0.0;
  h_opt->max       = 0.4;
  h_opt->r_min     = 0.6;
  h_opt->r_max     = 1.4;
  h_opt->text[0] = {0.2,0.75,"Run2 R22"};

  fin1 >> histofile;
  fin2 >> histofile;
  cout << "Histogram file name : " << histofile << endl;

  // get the mean and std values from data file
  for(int j=0; j<m; j++){

    int xenon, argon, ba,bc,eaa,eab,eca,ecb;

    for(int i=0; i<n; i++){
      fin1 >> key1;
      fin2 >> key2;
      argon = key1.find("Ar");
      xenon = key1.find("Xe");
      ba    = key1.find("BA");
      bc    = key1.find("BC");
      eaa   = key1.find("EAA");
      eab   = key1.find("EAB");
      eca   = key1.find("ECA");
      ecb   = key1.find("ECB");
      fin1 >> mean1[i] >> meanError1[i] >> sigma1[i] >> sigmaError1[i];
      fin2 >> mean2[i] >> meanError2[i] >> sigma2[i] >> sigmaError2[i];
      if(sigmaError1[i]>0.2) sigmaError1[i]=0.2;
      if(sigmaError2[i]>0.2) sigmaError2[i]=0.2;
      occ[i] = 0.1/2 + i*0.1;
      occError[i] = 0.1/2;
    }

    gr1 = new TGraphErrors(n,occ,sigma1,occError,sigmaError1);
    gr1->SetName(key1.c_str());

    gr2 = new TGraphErrors(n,occ,sigma2,occError,sigmaError2);
    gr2->SetName(key2.c_str());

    string gas = "(Ar)",text;
    if(xenon>0) gas = "(Xe)";
    if(ba>0)   text = "Barrel Side A "+gas;
    if(bc>0)   text = "Barrel Side C "+gas;
    if(eaa>0)  text = "Endcap Side A Wheel A "+gas;
    if(eab>0)  text = "Endcap Side A Wheel B "+gas;
    if(eca>0)  text = "Endcap Side C Wheel A "+gas;
    if(ecb>0)  text = "Endcap Side C Wheel B "+gas;

    h_opt->text[1] = {0.2,0.65,text};

    hist1 = trt_big_paper_wrapper(gr1);
    hist2 = trt_big_paper_wrapper(gr2);

    string pngfile  = "PngResiduals/ressigma"+to_string(10+j);
    h_opt->filename = pngfile;
    TrtBigPaperPlot(hist1, hist2, h_opt);

  } // for j

  fin1.close();
  fin2.close();

} // EOF
